@@ -7,30 +7,37 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * along with this program; if not, write to:
  *
- * Copyright (C) 2003-2004 Josh Buhl
+ * Free Software Foundation
+ * 51 Franklin Street, Fifth Floor
+ * Boston, MA 02110-1301
+ * USA
+ *
+ * Copyright (C) 2003-2005 Josh Buhl
+ * Copyright (C) 2010 Christine Spang
  *
  * This was originally the fish.c file from the Gnome Fish Applet
  * which I've mangled into the glunarclock applet. -- Josh Buhl
  *
- * Original Copyright (C)  1998-2002 Free Software Foundation, Inc.
+ * Original Copyright (C) 1998-2002 Free Software Foundation, Inc.
  *
  * Authors:
- *      George Lebl  <jirka@5z.com> (Gnome Fish Applet)
- *      Mark McLoughlin <mark@skynet.ie> (Gnome Fish Applet)
- *      Josh Buhl <jbuhl@users.sourceforge.net>
+ *      George Lebl  <jirka at 5z dot com> (Gnome Fish Applet)
+ *      Mark McLoughlin <mark at skynet dot ie> (Gnome Fish Applet)
+ *      Josh Buhl <jbuhl at users dot sourceforge dot net>
+ *      Christine Spang <spang@mit.edu>
  */
 
 #include "glunarclock.h"
 #include "moondata.h"
 
+#include <gdk/gdkkeysyms.h>
+
 #define UPDATE_APPLET_INTERVAL 1	/* seconds */
 
 #define FALLBACK_MOONIMAGE_FILE "glunarclock/moon_56frames.png"
@@ -41,6 +48,7 @@
 static void update_moon(MoonApplet * moon);
 static void update_image_number(MoonApplet * moon);
 static void compose_image   (MoonApplet *moon);
+static gchar *default_moon_image   (void);
 static gboolean load_moon_image (MoonApplet *moon);
 static GType moon_applet_get_type (void);
 static GObjectClass *parent_class;
@@ -70,32 +78,84 @@ void
 show_help (MoonApplet *moon, const char *link_id)
 {
 	GError *error = NULL;
+	char   *uri;
 
-	egg_screen_help_display (
-		gtk_widget_get_screen (GTK_WIDGET (moon)),
-		"glunarclock-applet-2", link_id, &error);
+#define MOON_HELP_DOC "glunarclock-applet-2"
 
-	if (error) {
-		show_error(_("There was an error displaying help: %s"),
-			   error->message);
-		
+	if (link_id)
+		uri = g_strdup_printf ("ghelp:%s?%s", MOON_HELP_DOC, link_id);
+	else
+		uri = g_strdup_printf ("ghelp:%s", MOON_HELP_DOC);
+
+	g_print("trying to show uri: %s \n", uri);
+
+	gtk_show_uri(gtk_widget_get_screen (GTK_WIDGET (moon)),	uri,
+			gtk_get_current_event_time(), &error);
+
+
+
+	g_free(uri);
+
+	if (error &&
+	    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 		g_error_free (error);
+	else if (error) {
+		GtkWidget *dialog;
+		char      *primary;
+
+		primary = g_markup_printf_escaped (
+				_("Could not display help document '%s'"),
+				MOON_HELP_DOC);
+		dialog = gtk_message_dialog_new (
+				NULL,
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_CLOSE,
+				"%s", primary);
+
+		gtk_message_dialog_format_secondary_text (
+					GTK_MESSAGE_DIALOG (dialog),
+					"%s", error->message);
+
+		g_error_free (error);
+		g_free (primary);
+
+		g_signal_connect (dialog, "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+
+		gtk_window_set_icon_name (GTK_WINDOW (dialog), MOON_ICON);
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (moon)));
+		/* we have no parent window */
+		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_title (GTK_WINDOW (dialog),
+				      _("Error displaying help document"));
+
+		gtk_widget_show (dialog);
+
 	}
+
 }
 
+/* moon_file_chooser callback */
 static void
 image_value_changed (GtkEntry   *entry,
 		     MoonApplet *moon)
-{	char *path;
+{
+	char *path = gtk_file_chooser_get_filename (
+			GTK_FILE_CHOOSER (moon->image_chooser));
 
-	path = gnome_file_entry_get_full_path (
-			GNOME_FILE_ENTRY (moon->pixmap_entry), TRUE);
 	if (!path || !path [0]) {
+		/* If the chosen file doesn't exist, don't do anything */
 		g_free (path);
 		return;
 	}
 
+	/* Update the GConf configuration value */
 	panel_applet_gconf_set_string (PANEL_APPLET (moon), "image", path, NULL);
+	/* Update the image preview */
+	gtk_image_set_from_file (moon->image_preview, path);
 
 	g_free (path);
 }
@@ -131,9 +191,9 @@ static void
 longitude_value_changed (GtkSpinButton *button,
 			MoonApplet    *moon)
 { 
-        panel_applet_gconf_set_float (
-			PANEL_APPLET (moon), "longitude",
-			gtk_spin_button_get_value_as_float (button), NULL);
+	panel_applet_gconf_set_float (
+		PANEL_APPLET (moon), "longitude",
+		gtk_spin_button_get_value_as_float (button), NULL);
 }
 
 static void
@@ -154,14 +214,14 @@ flip_value_changed (GtkToggleButton *toggle,
 			gtk_toggle_button_get_active (toggle), NULL);
 }
 
-static gboolean
+static void
 delete_event (GtkWidget  *widget,
 	      MoonApplet *moon)
 {
 	gtk_widget_hide (widget);
 }
 
-static gboolean
+static void
 close_dialog (GtkWidget  *widget,
 	      MoonApplet *moon)
 {
@@ -195,12 +255,11 @@ display_about_dialog (BonoboUIComponent *uic,
 		      MoonApplet        *moon,
 		      const char        *verbname)
 {
-	char        *authors [6];
-	GdkPixbuf   *pixbuf;
-	GError      *error = NULL;
-	char        *file;
+	gchar       *artists [2];
+	gchar       *authors [6];
+	gchar       *documenters [2];
 
-	const char *about = _("GNOME Lunar Clock displays the current phase of the moon "
+	const char *about = _("Lunar Clock displays the current phase of the moon "
 			      "and pertinent astronomical data.");
 
 	if (moon->about_dialog) {
@@ -211,47 +270,44 @@ display_about_dialog (BonoboUIComponent *uic,
 	}
 
 	authors [0] =  _("Josh Buhl <jbuhl@users.sourceforge.net>");
-	authors [1] =  _("Mike Henderson: wmMoonClock");
-	authors [2] =  _("Wanda & George: Applet Stuff");
-	authors [3] =  _("Nissen: cartoon moon graphics");
-	authors [4] =  _("Lots of friendly translators: po files");
+	authors [1] =  _("Christine Spang <spang@mit.edu>");
+	authors [2] =  _("Mike Henderson: wmMoonClock");
+	authors [3] =  _("Wanda & George: Applet Stuff");
+	authors [4] = _("Jon Ciesla: Release Technician & Project Admin");
 	authors [5] =  NULL;
 
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
-					   "glunarclock-logo.png", FALSE, NULL);
+	artists [0] = _("Cartoon moon graphics by Nissen http://ramnet.se/~nisse");
+	artists [1] = NULL;
 
-	pixbuf = gdk_pixbuf_new_from_file (file, &error);
-	g_free (file);
+ 	documenters [0] = "Josh Buhl <jbuhl@users.sourceforge.net>"; 
+ 	documenters [1] = NULL; 
 
-	if (error) {
-		g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
-		g_error_free (error);
-	}
+	gchar* website = "http://glunarclock.sourceforge.net";
+	gchar* website_label = "Latest News, etc. at the glunarclock website";
+	
+	gchar* translator_credits = "Ricardo Mones Lastra (ast_ES) \
+, Marquinos (ast), Vasif İsmayıloğlu (az), Vavor Dogonov (bg), J. Victor Martins (br), Keld Simonsen, Christian Rose, Joe Hansen (da), Josh Buhl, Roland Illig (de), Ricardo Mones Lastra, Victor Valdes Marquez (es, ast_es), Lauri Nurmi (fi), Roy Mathieu (fr), Kevin Patrick Scannell (ga), Jesús Bravo Álvarez (gl), Emese Kovacs, Horváth Árpád (hu), Andhika Padmawan (id), Yoshito Komatsu (ja), Rihards Prieditis (lv), Hasbullah Bin Pit (ms), Elros Cyriatan, Taco Witte, Erwin Poeze (nl), Kjartan Maraas (no), Joao Victor Duarte Martins (pt_BR), Laurentiu Buzdugan (ro), Yuriy Movchan (ru), Steven Murphy (rw), Andrej Kacian.sk (sk), Aleksandar Jelenak (sr), Christian Rose, Daniel Nylander (sv), pclouds, Clytie Siddall (vi), Pablo Saratxaga (vi, wa), Clytie Siddall (vi), Ji ZhengYu (zh_CN), Wei-Lun Chao (zh_TW)";
 
-	moon->about_dialog =
-		gnome_about_new (_("GNOME Lunar Clock"),
-				 VERSION,
-				 "Copyright \xc2\xa9 1999-2005 Josh Buhl, et al.",
-				 (const char *) about,
-				 (const char **) authors,
-				 NULL,
-				 NULL,
-				 pixbuf);
+	gchar* license = "This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. You should have received a copy of the GNU General Public License along with this program; if not, write to:  Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA";
+ 
 
-	if (pixbuf)
-		gdk_pixbuf_unref (pixbuf);
+	gtk_show_about_dialog(NULL,
+		     "name", PACKAGE,
+		     "version", VERSION,
+		     "copyright", "Copyright \xc2\xa9 1999-2010 Josh Buhl, Christine Spang, et al.",
+			      "license", license, "wrap-license", 1,	      
+		     "comments", (const char *) about,
+		     "artists", (const char **) artists,
+		     "authors", (const char **) authors,
+		     "documenters", (const char **) documenters,
+		     "translator-credits", translator_credits,
+			      "website", website, "website-label", website_label,
+		     "logo-icon-name", MOON_ICON,
+		     NULL);
 
-	gtk_window_set_wmclass (GTK_WINDOW (moon->about_dialog), "moon", "Moon");
-	gtk_window_set_screen  (GTK_WINDOW (moon->about_dialog),
-			       gtk_widget_get_screen (GTK_WIDGET (moon)));
-	gnome_window_icon_set_from_file (GTK_WINDOW (moon->about_dialog),
-					 GNOME_ICONDIR"/glunarclock-logo.png");
-
-	g_signal_connect (moon->about_dialog, "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &moon->about_dialog);
-
-	gtk_widget_show (moon->about_dialog);
+/* 	g_free (artists); */
+/* 	g_free (authors); */
+/* 	g_free (documenters); */
 }
 
 static void 
@@ -259,8 +315,8 @@ display_preferences_dialog (BonoboUIComponent *uic,
 			    MoonApplet        *moon,
 			    const char        *verbname)
 {
-	GladeXML  *xml;
 	GtkWidget *button;
+	GError *error = NULL;
 
 	if (moon->preferences_dialog) {
 		gtk_window_set_screen (GTK_WINDOW (moon->preferences_dialog),
@@ -269,10 +325,14 @@ display_preferences_dialog (BonoboUIComponent *uic,
 		return;
 	}
 
-	xml = glade_xml_new (MOON_GLADEDIR "/glunarclock.glade",
-			     "moon_preferences_dialog", NULL);
+	GtkBuilder *builder = gtk_builder_new();
+	if (!gtk_builder_add_from_file (builder, MOON_BUILDERDIR "/glunarclock.ui", &error)) {
+		g_warning ("Couldn't load builder file: %s", error->message);
+		g_error_free (error);
+	}
 
-	moon->preferences_dialog = glade_xml_get_widget (xml, "moon_preferences_dialog");
+	moon->preferences_dialog = GTK_WIDGET(
+			gtk_builder_get_object (builder, "moon_preferences_dialog"));
 
 	g_object_add_weak_pointer (G_OBJECT (moon->preferences_dialog),
 				   (void**) &moon->preferences_dialog);
@@ -283,23 +343,29 @@ display_preferences_dialog (BonoboUIComponent *uic,
 	gtk_dialog_set_default_response (GTK_DIALOG (moon->preferences_dialog),
 					 GTK_RESPONSE_OK);
 
-	gnome_window_icon_set_from_file (GTK_WINDOW (moon->preferences_dialog),
-					 GNOME_ICONDIR"/glunarclock-logo.png");
+	gtk_window_set_icon_from_file (GTK_WINDOW (moon->preferences_dialog),
+					 GNOME_PIXMAPSDIR "/glunarclock-logo.png",
+					 NULL);
 
 
-	moon->pixmap_entry = glade_xml_get_widget (xml, "image_entry");
+	moon->image_preview = GTK_IMAGE (
+			gtk_builder_get_object (builder, "moon_image_preview"));
+	/* XXX should only display a certain frame? how to do this? */
+	gtk_image_set_from_file (moon->image_preview,
+			moon->image);
 
-	moon->image_entry = gnome_file_entry_gtk_entry (
-		GNOME_FILE_ENTRY (moon->pixmap_entry));
+	moon->image_chooser = GTK_WIDGET (
+			gtk_builder_get_object (builder, "moon_file_chooser"));
+	gtk_file_chooser_set_filename(
+			GTK_FILE_CHOOSER (moon->image_chooser),
+			moon->image);
 
-	gtk_entry_set_text (GTK_ENTRY (moon->image_entry), moon->image);
-
-	g_signal_connect (moon->image_entry, "changed",
+	g_signal_connect (G_OBJECT (moon->image_chooser), "file_set",
 			  G_CALLBACK (image_value_changed), moon);
 
 
-
-	moon->frames_spin = glade_xml_get_widget (xml, "frames_spin");
+	moon->frames_spin = GTK_WIDGET(
+			gtk_builder_get_object (builder, "frames_spin"));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (moon->frames_spin),
 				   moon->n_frames);
 
@@ -308,7 +374,8 @@ display_preferences_dialog (BonoboUIComponent *uic,
 
 
 
-	moon->latitude_spin = glade_xml_get_widget (xml, "latitude_spin");
+	moon->latitude_spin = GTK_WIDGET(
+			gtk_builder_get_object (builder, "latitude_spin"));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (moon->latitude_spin),
 				   moon->latitude);
 
@@ -317,7 +384,8 @@ display_preferences_dialog (BonoboUIComponent *uic,
 
 
 
-	moon->longitude_spin = glade_xml_get_widget (xml, "longitude_spin");
+	moon->longitude_spin = GTK_WIDGET(
+			gtk_builder_get_object (builder, "longitude_spin"));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (moon->longitude_spin),
 				   moon->longitude);
 
@@ -325,33 +393,38 @@ display_preferences_dialog (BonoboUIComponent *uic,
 			  G_CALLBACK (longitude_value_changed), moon);
 
 
-	moon->flip_toggle = glade_xml_get_widget (xml, "flip_toggle");
+	moon->flip_toggle = GTK_WIDGET(
+			gtk_builder_get_object (builder, "flip_toggle"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (moon->flip_toggle), 
-				      moon->flip);
+					  moon->flip);
 
 	g_signal_connect (moon->flip_toggle, "toggled",
 			  G_CALLBACK (flip_value_changed), moon);
 
 
-	moon->lat_radio = glade_xml_get_widget (xml, "latitude_north");
+	moon->lat_radio = GTK_WIDGET(
+			gtk_builder_get_object (builder, "latitude_north"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (moon->lat_radio), 
-				      moon->is_north);
+					  moon->is_north);
 
-	button = glade_xml_get_widget (xml, "latitude_south");
+	button = GTK_WIDGET(
+			gtk_builder_get_object (builder, "latitude_south"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), 
-				      !(moon->is_north));
+					  !(moon->is_north));
 
 	g_signal_connect (G_OBJECT(moon->lat_radio), "toggled",
 			  G_CALLBACK (ns_value_changed), moon);
 
 
-	moon->long_radio = glade_xml_get_widget (xml, "longitude_east");
+	moon->long_radio = GTK_WIDGET(
+			gtk_builder_get_object (builder, "longitude_east"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (moon->long_radio), 
-				      moon->is_east);
+					  moon->is_east);
 
-	button = glade_xml_get_widget (xml, "longitude_west");
+	button = GTK_WIDGET(
+			gtk_builder_get_object (builder, "longitude_west"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), 
-				      !(moon->is_east));
+					  !(moon->is_east));
 
 	g_signal_connect (G_OBJECT(moon->long_radio), "toggled",
 			  G_CALLBACK (ew_value_changed), moon);
@@ -363,18 +436,24 @@ display_preferences_dialog (BonoboUIComponent *uic,
 	g_signal_connect (moon->preferences_dialog, "response",
 			  G_CALLBACK (handle_response), moon);
 
-	button = glade_xml_get_widget (xml, "done_button");
-        g_signal_connect (button, "clicked", G_CALLBACK (close_dialog), moon);
+	button = GTK_WIDGET(
+			gtk_builder_get_object (builder, "done_button"));
+	g_signal_connect (button, "clicked", G_CALLBACK (close_dialog), moon);
 
 	gtk_window_set_screen (GTK_WINDOW (moon->preferences_dialog),
-			       gtk_widget_get_screen (GTK_WIDGET (moon)));
-
+				   gtk_widget_get_screen (GTK_WIDGET (moon)));
+/*maybe resizing is desirable for viewing long moon image files */
+/* 	gtk_window_set_resizable (GTK_WINDOW (moon->preferences_dialog), FALSE); */
 	gtk_window_present (GTK_WINDOW (moon->preferences_dialog));
 
-	g_object_unref (xml);
+	g_object_unref (builder);
 }
 
 
+/* If the any of glunarclock's GConf values are changed via a GConf editor,
+ * update it in the running program as well. This also gets triggered
+ * when we save changes to gconf ourselves, unfortunately.
+ */
 static void
 image_changed_notify (GConfClient *client,
 		      guint        cnxn_id,
@@ -382,7 +461,6 @@ image_changed_notify (GConfClient *client,
 		      MoonApplet  *moon)
 {
 	const char *value;
-	char      *path = NULL;
 
 	if (!entry->value || entry->value->type != GCONF_VALUE_STRING)
 		return;
@@ -392,16 +470,25 @@ image_changed_notify (GConfClient *client,
 	if (!value [0] || (moon->image && !strcmp (moon->image, value)))
 		return;
 
-	if (moon->image)
-		g_free (moon->image);
+	/* Don't carry over non-absolute values from GConf */
+	if (g_path_is_absolute (value)) {
+		if (moon->image)
+			g_free (moon->image);
+		moon->image = g_strdup (value);
+		// printf("image_changed_notify: updating file chooser value to %s\n", default_moon_image());
+		if (moon->image_chooser != NULL)
+			gtk_file_chooser_set_filename (
+					GTK_FILE_CHOOSER (moon->image_chooser),
+					default_moon_image());
+	}
+	else {
+		/* Write the GConf value to be the default */
+		panel_applet_gconf_set_string (PANEL_APPLET (moon),
+				"image", default_moon_image(), NULL);
+		value = default_moon_image();
+	}
 
-	moon->image = g_strdup (value);
-
-	path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					  ALTERNATIVE_MOONIMAGE_FILE, FALSE, NULL);
-
-	if ((!strcmp(moon->image,ALTERNATIVE_MOONIMAGE_FILE)) ||
-	    (!strcmp(moon->image, path))) {
+	if (g_str_has_suffix(moon->image, ALTERNATIVE_MOONIMAGE_FILE)) {
 		moon->rotation_type = MIRROR;
 		panel_applet_gconf_set_int (
 			PANEL_APPLET (moon), "frames",
@@ -410,27 +497,30 @@ image_changed_notify (GConfClient *client,
 	else
 		moon->rotation_type = ROTATE;
 
-	g_free(path);
-
-	path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					  FALLBACK_MOONIMAGE_FILE, FALSE, NULL);
-
-	if ((!strcmp(moon->image,FALLBACK_MOONIMAGE_FILE)) ||
-	    (!strcmp(moon->image, path))) {
+	if (g_str_has_suffix(moon->image, FALLBACK_MOONIMAGE_FILE)) {
 		panel_applet_gconf_set_int (
 			PANEL_APPLET (moon), "frames",
 			FALLBACK_MOONIMAGE_FRAMES, NULL);
 	}
 
-	g_free(path);
-
 	update_image_number(moon);
 	load_moon_image (moon);
 	compose_image (moon);
 
-	if (moon->image_entry &&
-	    strcmp (gtk_entry_get_text (GTK_ENTRY (moon->image_entry)), moon->image))
-		gtk_entry_set_text (GTK_ENTRY (moon->image_entry), moon->image);
+	/* Update the chooser if it exists and we can get the name of the
+	 * currently selected file. This will automatically update the preview. */
+	gchar *filename = gtk_file_chooser_get_filename (
+			GTK_FILE_CHOOSER (moon->image_chooser));
+	if (moon->image_chooser != NULL && filename != NULL) {
+		printf("filename is: %s\n", filename);
+		if (moon->image_chooser && strcmp (filename, moon->image)) {
+			/* XXX TODO: ignores error (returns FALSE on error) */
+			gtk_file_chooser_set_filename (
+					GTK_FILE_CHOOSER (moon->image_chooser), filename);
+			printf("image_changed_notify 2: updating file chooser value to %s\n", default_moon_image());
+		}
+	}
+	g_free(filename);
 }
 
 static void
@@ -618,7 +708,7 @@ set_tooltip (MoonApplet *moon)
 
 	g_object_set_data (G_OBJECT (moon), "tooltips", tooltips);
 
-	tooltip = (visible(moon)) ?
+	tooltip = (visible()) ?
 	    g_strdup_printf(_("The moon is currently above the horizon."))
 	    :
 	    g_strdup_printf(_("The moon is currently below the horizon."));
@@ -666,7 +756,7 @@ update_moon(MoonApplet * moon)
 	gint old_image_number;
 
 	old_image_number = moon->image_number;
-	old_visible = visible(moon);
+	old_visible = visible();
 
 	update_moondata(moon);
 	update_image_number(moon);
@@ -682,7 +772,7 @@ update_moon(MoonApplet * moon)
 		gtk_widget_queue_draw(moon->drawing_area);
 	}
 
-	if(old_visible != visible(moon)) {
+	if(old_visible != visible()) {
 		destroy_tooltip(moon);
 		set_tooltip(moon);
 }
@@ -850,36 +940,22 @@ load_moon_image (MoonApplet *moon)
 	if (!moon->image)
 		return FALSE;
 
-	if (g_path_is_absolute (moon->image))
-		path = g_strdup (moon->image);
-	else {
-		path = gnome_program_locate_file (
-					NULL, GNOME_FILE_DOMAIN_PIXMAP,
-					moon->image, FALSE, NULL);
-		if (!path)
-			path = gnome_program_locate_file (
-					NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					moon->image, FALSE, NULL);
+	path = g_strdup (moon->image);
+
+	pixbuf = gdk_pixbuf_new_from_file (path, &error);
+	if (error) {
+		g_warning ("Cannot load '%s': %s", path, error->message);
+		show_error(_("There was an error loading moon image: %s"),
+				error->message);
+		g_error_free(error);
 	}
 
-	if (!path) {
-		g_warning ("Cannot locate '%s'", moon->image);
-		show_error("Cannot locate '%s'", moon->image);
-	} else {
-		pixbuf = gdk_pixbuf_new_from_file (path, &error);
-		if (error) {
-			g_warning ("Cannot load '%s': %s", path, error->message);
-			show_error(_("There was an error loading moon image: %s"),
-				   error->message);
-			g_error_free(error);
-		}
-
-		g_free (path);
-	}
+	g_free (path);
 
 	if (pixbuf) {
-		guchar *frame_rgb, *rgb;
-		gint width, height, xsrc, i, rowstride;
+/* 		guchar *frame_rgb, *rgb; */
+/* 		gint i; */
+		gint width, height, xsrc, rowstride;
 		gboolean has_alpha;
 
 		width = gdk_pixbuf_get_width(pixbuf) / moon->n_frames;
@@ -959,18 +1035,13 @@ static void
 compose_image (MoonApplet *moon)
 {
 	GtkWidget *widget = moon->drawing_area;
-	GdkPixbuf *composite_pb, *bg_pb;
-	GdkGC     *gc;
+	GdkPixbuf *composite_pb;
 	int        width  = -1;
 	int        height = -1;
-	int        rowstride = 4;
 	int        pixbuf_width = -1;
 	int        pixbuf_height = -1;
 	gboolean   rotate = FALSE;
 	gdouble    scale = 1.0;
-	guchar    *rgb;
-
-	int i;
 
 	if (!GTK_WIDGET_REALIZED (widget) ||
 	    widget->allocation.width <= 0 ||
@@ -1165,12 +1236,20 @@ handle_button_press (GtkWidget      *widget,
 		     GdkEventButton *event,
 		     MoonApplet     *moon)
 {
-	if (event->button != 1)
-		return FALSE; 
+	if ((event == NULL) || (event->button != 1))
+		return FALSE;
 
-	display_moondata_dialog (moon);
+	if (event->type == GDK_BUTTON_PRESS) {
+		if (!moon->moondata_dialog ||
+				!GTK_WIDGET_VISIBLE (moon->moondata_dialog))
+			display_moondata_dialog (moon);
+		else {
+			gtk_widget_hide (GTK_WIDGET (moon->moondata_dialog));
+		}
+		return TRUE;
+	}
 
-	return TRUE; 
+	return TRUE;
 }
 
 static void
@@ -1221,20 +1300,25 @@ setup_moon_widget (MoonApplet *moon)
 	gtk_widget_show_all (widget);
 }
 
+/* Right-click applet menu */
 static const BonoboUIVerb moon_menu_verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("Preferences", display_preferences_dialog),
 	BONOBO_UI_UNSAFE_VERB ("Help",        display_help_dialog),
 	BONOBO_UI_UNSAFE_VERB ("About",       display_about_dialog),
-
-        BONOBO_UI_VERB_END
+	BONOBO_UI_VERB_END
 };
 
+static gchar*
+default_moon_image (void)
+{
+	return g_strjoin ("/", GNOME_PIXMAPSDIR, FALLBACK_MOONIMAGE_FILE, NULL );
+}
+
 static gboolean
 moon_applet_fill (MoonApplet *moon)
 {
 	PanelApplet *applet = (PanelApplet *) moon;
 	GError      *error = NULL;
-	char      *path = NULL;
 
 	moon->orientation = panel_applet_get_orient (applet);
 
@@ -1249,17 +1333,21 @@ moon_applet_fill (MoonApplet *moon)
 		g_error_free (error);
 		error = NULL;
 	}
-	if (!moon->image)
-		moon->image = g_strdup ( FALLBACK_MOONIMAGE_FILE ); /* Fallback */
 
-	path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					  ALTERNATIVE_MOONIMAGE_FILE, FALSE, NULL);
+	/* We now *require* moon->image to be absolute. */
+	if (!moon->image || !g_path_is_absolute(moon->image)) {
+		if (moon->image_chooser != NULL)
+			printf("moon_applet_fill: updating file chooser value to %s\n", default_moon_image());
+			gtk_file_chooser_set_filename (
+					GTK_FILE_CHOOSER (moon->image_chooser),
+					default_moon_image());
+		moon->image = default_moon_image();
+	}
 
-	if((!strcmp (moon->image,ALTERNATIVE_MOONIMAGE_FILE)) || (!strcmp(moon->image, path)))
+	if (g_str_has_suffix (moon->image, ALTERNATIVE_MOONIMAGE_FILE))
 		moon->rotation_type = MIRROR;
 	else
 		moon->rotation_type = ROTATE;
-	g_free(path);
 
 	moon->n_frames = panel_applet_gconf_get_int (applet, "frames", &error);
 	if (error) {
@@ -1267,18 +1355,14 @@ moon_applet_fill (MoonApplet *moon)
 		g_error_free (error);
 		error = NULL;
 	}
-	path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					  FALLBACK_MOONIMAGE_FILE, FALSE, NULL);
-	if (moon->n_frames <= 0	&& ((!strcmp(moon->image, path)) ||
-				    (!strcmp (moon->image, FALLBACK_MOONIMAGE_FILE ))))
+
+	if (moon->n_frames <= 0	&& g_str_has_suffix ( moon->image, FALLBACK_MOONIMAGE_FILE))
 		moon->n_frames =  FALLBACK_MOONIMAGE_FRAMES;
 	else if (moon->n_frames <= 0 && (moon->rotation_type == MIRROR))
 		moon->n_frames = ALTERNATIVE_MOONIMAGE_FRAMES;
 	else if (moon->n_frames <= 0)
 		moon->n_frames = 20; /* Fallback */
 
-	g_free(path);
-
 	moon->latitude = panel_applet_gconf_get_float (applet, "latitude", &error);
 	if (error) {
 		g_warning ("Error getting 'latitude' preference: %s", error->message);
@@ -1439,8 +1523,8 @@ moon_applet_instance_init (MoonApplet      *moon,
 	moon->about_dialog = NULL;
 
 	moon->preferences_dialog = NULL;
-	moon->pixmap_entry       = NULL;
-	moon->image_entry        = NULL;
+	moon->image_preview      = NULL;
+	moon->image_chooser      = NULL;
 	moon->frames_spin        = NULL;
 	moon->longitude_spin     = NULL;
 	moon->latitude_spin      = NULL;
@@ -1494,6 +1578,7 @@ moon_applet_get_type (void)
 	return type;
 }
 
+/* applet factory */
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_GLunarclockApplet_Factory",
 			     moon_applet_get_type (),
 			     "glunarclock-applet-2",
