@@ -22,11 +22,8 @@
 #  include <config.h>
 #endif
 
-#include <gnome.h>
 #include <time.h>
 #include <X11/X.h>		/* need this for CurrentTime */
-#include <glade/glade-xml.h>
-#include <glade/glade.h>
 
 #include "MoonRise.h"
 #include "moondata.h"
@@ -219,14 +216,14 @@ update_moondata(MoonApplet * moon)
 	mod_time(&moondata.LAT);
 }
 
-static gboolean
+static void
 delete_event (GtkWidget  *widget,
 	      MoonApplet *moon)
 {
 	gtk_widget_hide (widget);
 }
 
-static gboolean
+static void
 close_data_dialog (GtkWidget  *widget,
 		   MoonApplet *moon)
 {
@@ -280,10 +277,10 @@ update_moondata_dialog(MoonApplet * moon)
 /* 	gtk_label_set_text(GTK_LABEL(mw.ilm), buf); */
 /* 	format_time(buf, size, moondata.NewMoon*24.0); */
 
-	g_snprintf(buf, size, _("%2d%"),(gint)moondata.FullMoon);
+	g_snprintf(buf, size, "%2d",(gint)moondata.FullMoon);
 	gtk_label_set_text(GTK_LABEL(mw.flm), buf);
 
-	g_snprintf(buf, size, "%2d%",(gint)(moondata.NewMoon));
+	g_snprintf(buf, size, "%2d",(gint)(moondata.NewMoon));
 	gtk_label_set_text(GTK_LABEL(mw.nwm), buf);
 
 /*  	g_snprintf(buf, size, _("%8.4f Hours"), moondata.NewMoon*24.0); */
@@ -337,42 +334,45 @@ update_moondata_dialog(MoonApplet * moon)
 }
 
 static void
-set_moonwidgets(GladeXML * xml)
+set_moonwidgets(GtkBuilder *builder)
 {
 
-	mw.lst = glade_xml_get_widget (xml, "lst");
-	mw.lmt = glade_xml_get_widget (xml, "lmt");
-	mw.lat = glade_xml_get_widget (xml, "lat");
-	mw.lsdt= glade_xml_get_widget (xml, "lsdt");
-	mw.ut  = glade_xml_get_widget (xml, "ut");
+	mw.lst = GTK_WIDGET ( gtk_builder_get_object (builder, "lst"));
+	mw.lmt = GTK_WIDGET ( gtk_builder_get_object (builder, "lmt"));
+	mw.lat = GTK_WIDGET ( gtk_builder_get_object (builder, "lat"));
+	mw.lsdt= GTK_WIDGET ( gtk_builder_get_object (builder, "lsdt"));
+	mw.ut  = GTK_WIDGET ( gtk_builder_get_object (builder, "ut"));
 
-/* 	mw.age = glade_xml_get_widget (xml, "age"); */
-	mw.frc = glade_xml_get_widget (xml, "frc");
-/* 	mw.ilm = glade_xml_get_widget (xml, "ilm"); */
-	mw.flm = glade_xml_get_widget (xml, "flm");
-	mw.nwm = glade_xml_get_widget (xml, "nwm");
-	mw.vis = glade_xml_get_widget (xml, "vis");
+/* 	mw.age = GTK_WIDGET (
+			gtk_builder_get_object (builder, "age")); */
+	mw.frc = GTK_WIDGET ( gtk_builder_get_object (builder, "frc"));
+/* 	mw.ilm = GTK_WIDGET (
+			gtk_builder_get_object (builder, "ilm")); */
+	mw.flm = GTK_WIDGET ( gtk_builder_get_object (builder, "flm"));
+	mw.nwm = GTK_WIDGET ( gtk_builder_get_object (builder, "nwm"));
+	mw.vis = GTK_WIDGET ( gtk_builder_get_object (builder, "vis"));
 
-	mw.uys = glade_xml_get_widget (xml, "uys");
-	mw.utd = glade_xml_get_widget (xml, "utd");
-	mw.utm = glade_xml_get_widget (xml, "utm");
-	mw.dys = glade_xml_get_widget (xml, "dys");
-	mw.dtd = glade_xml_get_widget (xml, "dtd");
-	mw.dtm = glade_xml_get_widget (xml, "dtm");
+	mw.uys = GTK_WIDGET ( gtk_builder_get_object (builder, "uys"));
+	mw.utd = GTK_WIDGET ( gtk_builder_get_object (builder, "utd"));
+	mw.utm = GTK_WIDGET ( gtk_builder_get_object (builder, "utm"));
+	mw.dys = GTK_WIDGET ( gtk_builder_get_object (builder, "dys"));
+	mw.dtd = GTK_WIDGET ( gtk_builder_get_object (builder, "dtd"));
+	mw.dtm = GTK_WIDGET ( gtk_builder_get_object (builder, "dtm"));
 
-	mw.ra  = glade_xml_get_widget (xml, "ra");
-	mw.dec = glade_xml_get_widget (xml, "dec");
-	mw.az  = glade_xml_get_widget (xml, "az");
-	mw.alt = glade_xml_get_widget (xml, "alt");
-	mw.dist= glade_xml_get_widget (xml, "dist");
+	mw.ra  = GTK_WIDGET ( gtk_builder_get_object (builder, "ra"));
+	mw.dec = GTK_WIDGET ( gtk_builder_get_object (builder, "dec"));
+	mw.az  = GTK_WIDGET ( gtk_builder_get_object (builder, "az"));
+	mw.alt = GTK_WIDGET ( gtk_builder_get_object (builder, "alt"));
+	mw.dist= GTK_WIDGET ( gtk_builder_get_object (builder, "dist"));
 
 }
 
 void 
 display_moondata_dialog (MoonApplet * moon)
 {
-	GladeXML  *xml;
 	GtkWidget *button;
+	GtkBuilder *builder = gtk_builder_new ();
+	GError *error = NULL;
 
 	if (moon->moondata_dialog) {
 		gtk_window_set_screen (GTK_WINDOW (moon->moondata_dialog),
@@ -381,27 +381,35 @@ display_moondata_dialog (MoonApplet * moon)
 		return;
 	}
 
-	xml = glade_xml_new (MOON_GLADEDIR "/glunarclock.glade", "moon_data_dialog", NULL);
-	moon->moondata_dialog = glade_xml_get_widget (xml, "moon_data_dialog");
+	if (!gtk_builder_add_from_file (builder, MOON_BUILDERDIR "/glunarclock.ui", &error)) {
+		g_warning ("Couldn't load builder file %s: %s",
+				MOON_BUILDERDIR "/glunarclock.ui", error->message);
+		g_error_free (error);
+	}
 
-	set_moonwidgets(xml);
+	moon->moondata_dialog = GTK_WIDGET(
+			gtk_builder_get_object (builder, "moon_data_dialog"));
+
+	set_moonwidgets(builder);
 
 /* 	gtk_window_set_wmclass (GTK_WINDOW (moon->moondata_dialog), "moon", "Moon"); */
-	
+
 	gtk_dialog_set_default_response (
 		GTK_DIALOG (moon->moondata_dialog), GTK_RESPONSE_CLOSE);
 
-	gnome_window_icon_set_from_file (
+	gtk_window_set_icon_from_file (
 		GTK_WINDOW (moon->moondata_dialog),
-		GNOME_ICONDIR"/glunarclock-logo.png");
-		
+		GNOME_PIXMAPSDIR "/glunarclock-logo.png",
+		NULL);
+
 	g_signal_connect (moon->moondata_dialog, "delete_event",
 			  G_CALLBACK (delete_event), moon);
 
 	g_signal_connect (moon->moondata_dialog, "response",
 			  G_CALLBACK (handle_data_response), moon);
 
-	button = glade_xml_get_widget (xml, "close_button");
+	button = GTK_WIDGET (
+			gtk_builder_get_object (builder, "moondata_close_button"));
         g_signal_connect (button, "clicked", G_CALLBACK (close_data_dialog), moon);
 
 	update_moondata_dialog (moon);
@@ -411,5 +419,5 @@ display_moondata_dialog (MoonApplet * moon)
 
 	gtk_window_present (GTK_WINDOW (moon->moondata_dialog));
 
-	g_object_unref (xml);
+	g_object_unref (builder);
 }
