@@ -26,8 +26,8 @@
 
 G_BEGIN_DECLS
 
-gboolean visible();
-gdouble moonphase();
+gboolean visible(void);
+gdouble moonphase(void);
 void update_moondata(MoonApplet * moon);
 void display_moondata_dialog(MoonApplet * moon);
 void update_moondata_dialog(MoonApplet * moon);
